
#ifndef _GDT_H_
#define _GDT_H_
#include <defs.h>
#include <stdtypes.h>
#include <memory.h>
// The general format of a segment descriptor, taken from GeekOS
typedef struct __gdtEntry {
	word_t sizeLow				 __attribute__((packed)) ;
	dword_t baseLow			: 24 __attribute__((packed)) ;
	dword_t type			: 4  __attribute__((packed)) ;
	dword_t system			: 1  __attribute__((packed)) ;
	dword_t dpl				: 2  __attribute__((packed)) ;
	dword_t present			: 1  __attribute__((packed)) ;
	dword_t sizeHigh		: 4  __attribute__((packed)) ;
	dword_t avail			: 1  __attribute__((packed)) ;
	dword_t reserved		: 1  __attribute__((packed)) ; // set to zero
	dword_t dbBit			: 1  __attribute__((packed)) ;
	dword_t granularity		: 1  __attribute__((packed)) ;
	half_t baseHigh				 __attribute__((packed)) ;
} gdtEntry;
#define gdtSetSize(g, sz) (g)->sizeLow = (sz & 0xFFFF); (g)->sizeHigh = ((sz >> 16) & 0x0F)
#define gdtSetBase(g, b) (g)->baseLow = (b & 0xFFFFFF); (g)->baseHigh = (b >> 24)
// This is defined in int.s, and needs moving somewhere else
// The parameter should consist of 16 bit base,
// followed by 32 bit base address, describing the IDT.
extern "C" void gdtLoad(word_t* limitAndBase);

// public functions from gdt.c
void InitGDT();	
void InitLDT();
gdtEntry* gdtGet(byte_t entry);
void gdtPut(byte_t entry, gdtEntry* desc);
void gdtRemove(byte_t entry);
gdtEntry* gdtGetFree(byte_t* entry);

extern "C" byte_t gdtAdd(byte_t type,
			ulong_t base = 0,
			ulong_t size = 0xFFFFF,
			byte_t system = 1,
			byte_t granularity = 1,
			byte_t dbBit = 1,
			byte_t dpl = 0);

void gdtSet(byte_t entry,
			byte_t type,
			ulong_t base = 0,
			ulong_t size = 0xFFFFF,
			byte_t system = 1,
			byte_t granularity = 1,
			byte_t dbBit = 1,
			byte_t dpl = 0);
#define GDTBASE(x) (((x)->baseLow)|((x)->baseHigh)<<24)

extern "C" dword_t getLinear (dword_t selector, dword_t offset);
#endif // !_GDT_H_

