
#ifndef _IDT_H_
#define _IDT_H_
#include <memory.h>
// We'll handle all possible interrupts.
#define IDT_NENTRIES 256

// Exceptions range from 0-17
#define IDT_FIRST_EXCEPTION 0
#define IDT_NUM_EXCEPTIONS 18

// External IRQs range from 32-47
#define IDT_FIRST_EXTERNAL 32
#define IDT_NUM_EXTERNAL 16

typedef struct __InterruptGate {
	word_t offsetLow;
	word_t segmentSelector;
	dword_t reserved : 5;
	dword_t signature : 8;
	dword_t dpl : 2;
	dword_t present : 1;
	word_t offsetHigh;
} InterruptGate;

typedef union __idtEntry {
	InterruptGate ig;
	// In theory we could have members for trap gates
	// and task gates if we wanted.
} idtEntry;

// This is defined in int.s
// The parameter should consist of 16 bit base,
// followed by 32 bit base address, describing the IDT.
extern "C" void idtLoad(word_t* limitAndBase);
// interrupt handler function
typedef void (*InterruptHandler)();

// global functions
void InitIDT();
void InstallHandler(byte_t i, InterruptHandler hand);

// irq stuff
void EnableIRQ(byte_t irq, bool_t enable);

#endif // !_GDT_H_

