#ifndef _KALLOC_H
#define _KALLOC_H
#include <stdtypes.h>
#include <printk.h>
#include <memory.h>
typedef struct {
	ptr_t page;
	int  npage;
	word_t sel;
}mem_t;

#ifdef __cplusplus
extern "C" {
	#undef memcpy
	#undef memset
	#undef memcmp
	#undef memzero
	ptr_t memcpy(ptr_t dest, cptr_t src, count_t count);
	ptr_t memset(ptr_t dest, byte_t c, count_t count);
	int_t memcmp(cptr_t p1, cptr_t p2, count_t count);
}
#define memzero(dest,count) memset((ptr_t)dest,(byte_t)0,(count_t)count)
#endif
void InitPG();
ptr_t pgAlloc();
void pgRelease(ptr_t pg);
void pgRelease_c(ptr_t pg,int npage);
ptr_t pgAlloc_c(count_t nPg);
enum privilege {KERNEL, SYSTEM, USER};
enum purpose {DATA, CODE,TSS};
byte_t installSegment(ptr_t base,size_t size,purpose dc,privilege ring);
void releaseSegment(byte_t sel);
void free_mem(mem_t*);
#define PAGE_SIZE          (0x1000)
// Start the bitmap immediately after the IDT, at 8k

// Default to a memory pool starting at 1MB and ending at 4MB
// This assumes we have at least 4MB of memory.

#define POOL_SIZE_IN_PAGES (POOL_SIZE/PAGE_SIZE)
#define BITMAP_SIZE        (POOL_SIZE_IN_PAGES >> 3)

#define PAGE_NO(x) ((((int)x)-POOL_START)/(PAGE_SIZE))
#define CHECK_BITMAP(n) (*((byte_t*)(BITMAP_START+(n>>3))))&(1<<(n&0x7))
#define CHECK_ALLOC(x) ((CHECK_BITMAP(PAGE_NO(x)))!=0)
#define PRINT_BITMAP 	printk("\n Bitmap :: ");\
	for(int _x=0;_x<BITMAP_SIZE;_x++)\
		printk("%x,",*((byte_t*)(BITMAP_START+_x)));

extern int _ctr_alloc;
extern int _ctr_release;
#define DEBUG_MEM(mem_ptr) printk("\n Mem details :%x,%d,%d",(long)((mem_ptr)->page),(mem_ptr)->npage,(mem_ptr)->sel);
#endif 
