/* tss.h
   declaration of TSS setup procedure                                         */

#ifndef TSS_H
#define TSS_H
#include <gdt.h>
#include <video.h>
#include <kalloc.h>
#include <stdtypes.h>
#include <asm.h>
#define TSS_ESP0 0x04
#define TSS_SS0  0x08
#define TSS_ESP1 0x0C
#define TSS_SS1  0x10
#define TSS_ESP2 0x14
#define TSS_SS2  0x18
#define TSS_ES   0x48
#define TSS_CS   0x4C
#define TSS_SS   0x50
#define TSS_DS   0x54
#define TSS_FS   0x58
#define TSS_GS   0x5C
#define TSS_IOMP 0x66
struct tss_struct {
	dword_t	back_link;	/* 16 high bits zero */
	dword_t	esp0;
	dword_t	ss0;		/* 16 high bits zero */
	dword_t	esp1;
	dword_t	ss1;		/* 16 high bits zero */
	dword_t	esp2;
	dword_t	ss2;		/* 16 high bits zero */
	dword_t	cr3;
	dword_t	eip;
	dword_t	eflags;
	dword_t	eax,ecx,edx,ebx;
	dword_t	esp;
	dword_t	ebp;
	dword_t	esi;
	dword_t	edi;
	dword_t	es;		/* 16 high bits zero */
	dword_t	cs;		/* 16 high bits zero */
	dword_t	ss;		/* 16 high bits zero */
	dword_t	ds;		/* 16 high bits zero */
	dword_t	fs;		/* 16 high bits zero */
	dword_t	gs;		/* 16 high bits zero */
	dword_t	ldt;		/* 16 high bits zero */
	dword_t	trace_bitmap;	/* bits: trace 0, bitmap 16-31 */
};
extern volatile int fromThread;
#define switch_to(sel) {\
struct {long a,b;} __tmp={0,0}; \
__asm__("movw %%dx,%1\n\t" \
	"ljmp %0\n\t" \
	"clts\n" \
	::"m" (*&__tmp.a),"m" (*&__tmp.b), \
	"d" (sel)); \
}
extern mem_t ktss;
extern mem_t threadTss;
extern void initThreadTSS();
extern void buildKernelTSS ();	
extern void buildThreadTSS(mem_t code,mem_t data,mem_t stack);
extern "C" void doDebuggerInt(dword_t);
#define TF_BIT 8
#define IF_BIT 9
#define SET_BIT(bitno,flags) \
	flags=(flags)|(1<<(bitno))
#define CLEAR_BIT(bitno,flags) \
	flags=(flags)&(~(1<<(bitno)))
#define THREAD_TSS ((struct tss_struct*)(threadTss.page))
#endif
