
#include <stdtypes.h>
#include <file.h>
#include <con_priv.h>
#include <video.h>
#include <sys.h>
#include <console.h>

// current position, attribute and number of rows/cols
byte_t cx;
byte_t cy;
byte_t at;
byte_t nr;
byte_t nc;

// video device
static fileh vid;

// simple putc at current location
void con_putc(char_t ch)
{
	// silently fail on bad position
	if (cy >= nr || cx >= nc)
		return;

	char_t rec[2];
	rec[0] = ch;
	rec[1] = at;

	write_R(vid, rec, (nc * cy) + cx);
	++cx;
	
	if (cx == nc)
	{
		cx = 0;
		++cy;
		if (cy == nr)
		{
			con_scroll();
		}
	}
}

// update the cursor position
void con_update_cursor()
{
	ioctl(vid, VID_SET_CURSOR, (ptr_t)((nc * cy) + cx));
}

// handle a backspace
void con_bs()
{
	if (cx)
	{
		char_t rec[2];
		rec[0] = ' ';
		rec[1] = at;

		--cx;
		write_R(vid, rec, (nc * cy) + cx);
	}
}

// handle a line feed
void con_lf()
{
	// pad to end of line
	while (cx < nc)
	{
		char_t rec[2];
		rec[0] = ' ';
		rec[1] = at;

		write_R(vid, rec, (nc * cy) + cx);
		cx++;
	}
	
	// start a new line
	cx = 0;
	++cy;
	if (cy == nr)
	{
		con_scroll();
	}
}

// handle a horizontal tab
void con_ht()
{
/*	count_t tomove = TAB_SPACES - (cx % TAB_SPACES);
	if ((cx + tomove) < nc)
	{
		while (tomove--)
		{
			char_t rec[2];
			rec[0] = ' ';
			rec[1] = at;

			write_R(vid, rec, (nc * cy) + cx);
			++cx;
		}
	}*/
}

// handle a vertical tab
void con_vt()
{
	// pad to end of line
	while (cx < nc)
	{
		char_t rec[2];
		rec[0] = ' ';
		rec[1] = at;

		write_R(vid, rec, (nc * cy) + cx);
		cx++;
	}
	
	// start a new line at current col
	++cy;
	if (cy == nr)
	{
		con_scroll();
	}
}

// scroll the console
void con_scroll()
{
	// call the scroll helper ioctl
	ioctl(vid, VID_SCROLL_HELPER, 0);
	
	// pad last line with spaces
	cx = 0;
	cy = nr - 1;
	while (cx < nc)
	{
		char_t rec[2];
		rec[0] = ' ';
		rec[1] = at;

		write_R(vid, rec, (nc * cy) + cx);
		cx++;
	}
	
	// move back to start of last line
	cx = 0;
}

// clear the console
void con_clear()
{
	cx = cy = 0;
	con_update_cursor();
	while (cy < nr)
	{
		while (cx < nc)
		{
			char_t rec[2];
			rec[0] = ' ';
			rec[1] = at;

			write_R(vid, rec, (nc * cy) + cx);
			++cx;
		}
		cx = 0;
		++cy;
	}
	cx = cy = 0;
}

// initialise the video output interface
void con_init_vid()
{
	// open the video device
	vid = OpenDevice(VIDEO_DEVICE);

	// get video parameters
	VidParams par;
	ioctl(vid, VID_GET_PARAMS, &par);
	nr = par.rows;
	nc = par.cols;
	
	// set initial attribute and clear the screen
	at = 0x07;		// white on black
	con_clear();
}

// handle escape sequences
count_t con_esc(string_t p, count_t sz)
{
	// currently we don't handle any escape sequences
	return 0;
}
