#include <stdtypes.h>
#include <file.h>
#include <device.h>
#include <console.h>
#include <con_priv.h>
#include <ascii.h>
#include <keyboard.h>

k_dev devC;
static fileh keyb;

//////////////////////////////////////////////////////////////
// Console device

int_t con_read(fileh f, char_t* ch)
{
	return read(keyb, ch);
}

int_t con_write(fileh f, char_t ch)
{
	con_out(&ch, 1);
	return DEV_SUCCESS;
}

int_t con_open(fileh f)
{
	return DEV_SUCCESS;
}

int_t con_close(fileh f)
{
	return DEV_SUCCESS;
}

int_t con_ioctl(fileh f, ulong_t request, void* arg)
{
	if(request==CON_CLEAR) con_clear();
	else return DEV_ERR_INVALID_REQUEST;
	return DEV_SUCCESS;
}

void InitConsole()
{
	keyb_init();
	con_init_vid();
	
	devC.flags = 0;
	devC.ulRecordSize = 0;
	
	devC.fnOpen = con_open;
	devC.fnClose = con_close;
	devC.fnRead = con_read;
	devC.fnWrite = con_write;
	devC.fnIoctl = con_ioctl;
	
	RegisterDevice(CONSOLE_DEVICE, &devC);
}

//////////////////////////////////////////////////////////////
// Output

// process an output queue
void con_out(cstring_t q, count_t sz)
{
	string_t p = (string_t)q;
	while (sz--)
	{
		switch (*p)
		{
			case ASCII_NUL:		// padding character
			
			case ASCII_SOH:		// all these aren't handled yet
			case ASCII_STX:
			case ASCII_ETX:
			case ASCII_EOT:
			case ASCII_ENQ:
			case ASCII_ACK:
			case ASCII_BEL:
			case ASCII_CR:
			case ASCII_SO:
			case ASCII_SI:
			case ASCII_DLE:
			case ASCII_DC1:
			case ASCII_DC2:
			case ASCII_DC3:
			case ASCII_DC4:
			case ASCII_NAK:
			case ASCII_SYN:
			case ASCII_ETB:
			case ASCII_CAN:
			case ASCII_EM:
			case ASCII_FS:
			case ASCII_GS:
			case ASCII_RS:
			case ASCII_US:
			case ASCII_FF:
				break;
				
			case ASCII_LF:		// line feed (new line)
				con_lf();
				break;
			case ASCII_BS:		// backspace
				con_bs();
				break;
			case ASCII_HT:		// horizontal tab
				con_ht();
				break;
			case ASCII_SUB:		// substitute, next char is outputted raw, eg ESC.
				if (sz > 1)		// dont kill ourself if the input string ends with a SUB.
				{
					con_putc(*(++p));
					--sz;
				}
				break;
			case ASCII_ESC:		// starts an escape sequence
				{
					count_t adj = con_esc(++p, sz--);
								// con_esc() can cope if it hits the end of the string
					p += adj;
					sz -= adj;
					break;
				}
			case ASCII_VT:		// vertical tab (new line at same col)
				con_vt();
				break;
			
			default:			// all other characters
				con_putc(*p);
		}
		++p;
	}
	
	// now we have finished, update the cursor location
	con_update_cursor();
}

//////////////////////////////////////////////////////////////
// Keyboard interface

// we hook this up to handle command keys
void keyb_hook(ulong_t hook, void* param)
{
	// no hooks yet
}

void keyb_init()
{
	keyb = OpenDevice(KEYBOARD_DEVICE);
	
	// setup our hook
	ioctl(keyb, KEYBOARD_SET_HOOK, keyb_hook);
	// set mode
	ioctl(keyb, KEYBOARD_SET_MODE,KEYBOARD_FULLY_COOKED);
}
