
#include <stdtypes.h>

#include <device.h>
#include <file.h>
#include <video.h>
#include <kalloc.h>

/////////////////////////////////////////////////////////////////////
// Device table

/*
	Structure of a device table entry:
		- 8 bytes
		- first 4 are device number
		- second 4 are k_dev*
*/

// pointer to the device table, which is after the memory bitmap
#define pDeviceTable ((ulong_t*)DEVICE_TABLE_START)

k_dev InvalidDevice;

// for now we just use 4k of memory, limiting us to 512 devices at any one time

// initialise the device table
void InitDeviceTable()
{
	VidWrite("Initialising device table... ");
	memset (((void*)pDeviceTable), (byte_t)0, 4096);
	memset(((void*)&InvalidDevice), 0, sizeof(InvalidDevice));	// by setting all to zero, we clear DEV_FLAG_VALID
	VidWrite("Done\n");
}

// lookup a device
k_dev* device_tbl_lookup(ulong_t dev)
{
	int i = 0;
	while (i < 512)
	{
		if (*(pDeviceTable + (2 * i)) == dev)
			return (k_dev*)(*(pDeviceTable + (2 * i) + 1));
		++i;
	}
	return &InvalidDevice;	// return the invalid device if not found
}

// add a device to the table
int_t device_tbl_add(ulong_t dev, k_dev* pDev)
{
	int i = 0;
	while (i < 512)
	{
		if (*(pDeviceTable + (2 * i)) == dev)
		{
			(*(pDeviceTable + (2 * i) + 1)) = (ulong_t)pDev;
			return DEV_SUCCESS;
		}
		
		if (*(pDeviceTable + (2 * i)) == 0)
		{
			(*(pDeviceTable + (2 * i))) = dev;
			(*(pDeviceTable + (2 * i) + 1)) = (ulong_t)pDev;
			return DEV_SUCCESS;
		}
		
		++i;
	}
	return DEV_ERR_TABLE_FULL;
}

// remove a device from the table
int_t device_tbl_remove(ulong_t dev)
{
	int i = 0;
	while (i < 512)
	{
		if (*(pDeviceTable + (2 * i)) == dev)
		{
			(*(pDeviceTable + (2 * i))) = 0;
			(*(pDeviceTable + (2 * i) + 1)) = 0;
			return DEV_SUCCESS;
		}
		++i;
	}
	return DEV_ERR_NO_ENTRY;
}

/////////////////////////////////////////////////////////////////////
// Public Functions

// open a device
fileh OpenDevice(ulong_t dev)
{
	fileh f;
	file_data* pF = get_free_file(&f);
	pF->dev = device_tbl_lookup(dev);
	(*(pF->dev->fnOpen))(f);
	
	return f;
}

// get a device without opening a file, useful occaisonally
k_dev* GetDevice(ulong_t dev)
{
	k_dev* pDev = device_tbl_lookup(dev);
	return pDev;
}

// register a device
int_t RegisterDevice(ulong_t dev, k_dev* pDev)
{
	return device_tbl_add(dev, pDev);
}

// unregister a device
int_t UnregisterDevice(ulong_t dev)
{
	return device_tbl_remove(dev);
}
