
#include <stdtypes.h>

#include <device.h>
#include <file.h>
#include <video.h>
#include <sys.h>
#include <asm.h>
#include <kalloc.h>

////////////////////////////////////////////////////////////
// File table

#define FILE_TABLE_END	(FILE_TABLE_START + FILE_TABLE_SIZE)
#define MAX_FILE			((FILE_TABLE_SIZE / sizeof(file_data)) - 1)

// we allocate 1k giving us jus over 100 files if sizeof(file_data) == 10

file_data* get_file_data(fileh f)
{
	if (f > MAX_FILE)
		panic("Bad File!");
		
	ulong_t ul = FILE_TABLE_START + f * sizeof(file_data);
	return (file_data*)ul;
}

void release_file(fileh f)
{
	file_data* data = get_file_data(f);
	memzero(data, sizeof(file_data));
	data->flags = FILE_FREE;
}

file_data* get_free_file(fileh* f)
{
	fileh i = 0;
	file_data* p;
	
	while (i < MAX_FILE)
	{
		ulong_t ul = FILE_TABLE_START + i * sizeof(file_data);
		p = (file_data*)ul;
		
		if (p->flags & FILE_FREE)
		{
			p->flags = 0;
			*f = i;
			return p;
		}
		
		++i;
	}
	
	panic("Out of files!");
	return NULL;	// unreachable
}

void InitFileTable()
{
	VidWrite("Initialising file table... ");
	fileh i = 0;
	memzero((void*)FILE_TABLE_START, FILE_TABLE_SIZE);
	
	while (i < MAX_FILE)
	{
		ulong_t ul = FILE_TABLE_START + i++ * sizeof(file_data);
		file_data* p = (file_data*)ul;
		p->flags = FILE_FREE;
	}
	
	PrintHex(MAX_FILE);
	VidWrite(" files ready.\n");
}

////////////////////////////////////////////////////////////
// Public Functions

int_t close(fileh f)
{
	file_data* pF = get_file_data(f);
	int_t ret = (*(pF->dev->fnClose))(f);
	if (ret != DEV_ABORT)
		release_file(f);
	return ret;
}

int_t write(fileh f, char_t ch)
{
	file_data* pF = get_file_data(f);
	return (*(pF->dev->fnWrite))(f, ch);
}

/*
	Interrupts are enabled during read(), a GPF occurs when reading from
	the keyboard.
	Just protecting DequeueKey() does not stop this error so I guess it
	has something to do with the call thru a function pointer being
	interrupted by a keypress.
	Help!
*/

int_t read(fileh f, char_t* ch)
{
	file_data* pF = get_file_data(f);
	cli();
	int_t ret = (*(pF->dev->fnRead))(f, ch);
	sti();
	return ret;
}

int_t write_R(fileh f, void* p, ulong_t rec)
{
	file_data* pF = get_file_data(f);
	return (*(pF->dev->fnWrite_R))(f, p, rec);
}

int_t read_R(fileh f, void* p, ulong_t rec)
{
	file_data* pF = get_file_data(f);
	return (*(pF->dev->fnRead_R))(f, p, rec);
}

// a macro is used to call ioctl
int_t ioctl2(fileh f, ulong_t request, void* param)
{
	file_data* pF = get_file_data(f);
	return (*(pF->dev->fnIoctl))(f, request, param);
}
