
#include <stdtypes.h>

#include <idt.h>
#include <defs.h>
#include <kalloc.h>
#include <video.h>
#include <asm.h>
#include <sys.h>

// size of IDT
#define IDT_SIZE (IDT_NENTRIES << 3)

// start the IDT at 6k

// the default IDT is 2k in size, so spans from 6k to 8k

extern "C" void DummyInterrupt();

// interrupts must be disabled when calling this
void PrivateInstallHandler(idtEntry* desc, InterruptHandler hand)
{
	iptr_t addr = (iptr_t)hand;
	int index=((long)((void*)desc)-IDT_START)>>3;	
	desc->ig.offsetLow = addr & 0xffff;
	desc->ig.segmentSelector = KERNEL_CS;
	desc->ig.reserved = 0;
	desc->ig.signature = 0x70; // == 01110000b
	desc->ig.dpl = (index<17)?3:0;
	desc->ig.present = 1;
	desc->ig.offsetHigh = addr >> 16;
}
extern "C" void MaskAllIrq();
// initialise the idt
extern "C" void DefInterrupt0();
extern "C" void DefInterrupt1();
extern "C" void DefInterrupt2();
extern "C" void DefInterrupt3();
extern "C" void DefInterrupt4();
extern "C" void DefInterrupt5();
extern "C" void DefInterrupt6();
extern "C" void DefInterrupt7();
extern "C" void DefInterrupt8();
extern "C" void DefInterrupt9();
extern "C" void DefInterrupt10();
extern "C" void DefInterrupt11();
extern "C" void DefInterrupt12();
extern "C" void DefInterrupt13();
extern "C" void DefInterrupt14();
extern "C" void DefInterrupt15();
extern "C" void DefInterrupt16();
extern "C" void DefInterrupt17();
extern "C" void DefInterrupt18();
extern "C" void DefInterrupt19();
// called by the dummy handler
extern "C" void DoDummyInterrupt()
{
	panic("**** really *** Bad Interrupt");
}
/*
 Print (ebp+12) to (ebp+24) in steps of 4 , to know where the interrupt occured
 See how linux handles these traps , cog the code from tehre .
 */
extern "C" void DefInt(int x)
{
	PrintHex(x);
	panic("*** Bad Interrupt");
}
void InitIDT()
{
    word_t limitAndBase[3];
	count_t i = 0;

	VidWrite("Initialising IDT... ");

    // Blank out the IDT before we start just for safety
	memset((ptr_t)IDT_START, (byte_t)0, IDT_SIZE);
	
	PrivateInstallHandler((idtEntry*)(IDT_START + (0 << 3)), &DefInterrupt0);
	PrivateInstallHandler((idtEntry*)(IDT_START + (1 << 3)), &DefInterrupt1);
	PrivateInstallHandler((idtEntry*)(IDT_START + (2 << 3)), &DefInterrupt2);
	PrivateInstallHandler((idtEntry*)(IDT_START + (3 << 3)), &DefInterrupt3);
	PrivateInstallHandler((idtEntry*)(IDT_START + (4 << 3)), &DefInterrupt4);
	PrivateInstallHandler((idtEntry*)(IDT_START + (5 << 3)), &DefInterrupt5);
	PrivateInstallHandler((idtEntry*)(IDT_START + (6 << 3)), &DefInterrupt6);
	PrivateInstallHandler((idtEntry*)(IDT_START + (7 << 3)), &DefInterrupt7);
	PrivateInstallHandler((idtEntry*)(IDT_START + (8 << 3)), &DefInterrupt8);
	PrivateInstallHandler((idtEntry*)(IDT_START + (9 << 3)), &DefInterrupt9);
	PrivateInstallHandler((idtEntry*)(IDT_START + (10 << 3)), &DefInterrupt10);
	PrivateInstallHandler((idtEntry*)(IDT_START + (11 << 3)), &DefInterrupt11);
	PrivateInstallHandler((idtEntry*)(IDT_START + (12 << 3)), &DefInterrupt12);
	PrivateInstallHandler((idtEntry*)(IDT_START + (13 << 3)), &DefInterrupt13);
	PrivateInstallHandler((idtEntry*)(IDT_START + (14 << 3)), &DefInterrupt14);
	PrivateInstallHandler((idtEntry*)(IDT_START + (15 << 3)), &DefInterrupt15);
	PrivateInstallHandler((idtEntry*)(IDT_START + (16 << 3)), &DefInterrupt16);
	PrivateInstallHandler((idtEntry*)(IDT_START + (17 << 3)), &DefInterrupt17);
	PrivateInstallHandler((idtEntry*)(IDT_START + (18 << 3)), &DefInterrupt18);
	PrivateInstallHandler((idtEntry*)(IDT_START + (19 << 3)), &DefInterrupt19);
	i=20;
	// load dummy interrupt handlers
	while (i < IDT_NENTRIES){
		PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), &DoDummyInterrupt);
		//PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), &(DefInterrupt[i]));
		i++;
	}

    // Cruft together a 16 bit limit and 32 bit base address
    // to load into the IDTR.
    limitAndBase[0] = IDT_SIZE;
    limitAndBase[1] = IDT_START & 0xffff;
    limitAndBase[2] = IDT_START >> 16;

    // Install the new table in the IDTR.
    idtLoad(limitAndBase);

	VidWrite("Done\n");
	MaskAllIrq();
}

// this should be kept up to date with what we load when booting
static word_t irqMask = 0xFFFB;

#define MASTER(mask) ( (mask) & 0xff )
#define SLAVE(mask) ( ((mask)>>8) & 0xff )

void EnableIRQ(byte_t irq, bool_t enable)
{
	byte_t oldMask, newMask, mask;

	if (enable)
		mask = irqMask & ~(1 << irq);
	else
		mask = irqMask | (1 << irq);

    oldMask = MASTER(irqMask);
    newMask = MASTER(mask);
    if ( newMask != oldMask )
	{
		outb(newMask, 0x21);
    }

    oldMask = SLAVE(irqMask);
    newMask = SLAVE(mask);
    if ( newMask != oldMask )
	{
		outb(newMask, 0xA1);
    }

    irqMask = mask;
}
// public function to install an interrupt handler,
//  interrupts must be disabled?
void InstallHandler(byte_t i, InterruptHandler hand)
{
	PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), hand);
}

