
#include <stdtypes.h>

#include <video.h>
#include <kalloc.h>
#include <asm.h>

///////////////////////////////////////////
// the old Vid* public functions, for outputing before the
// video & console drivers are loaded.

// current position and attribute
// private functions
void VidScroll();
void VidUpdateCursor();

#define TAB_SPACES 4
struct __cursor cursor;
///////////////////////////////////////////////////////////

// low level access to video memory
count_t VidWrite(cstring_t buffer)
{
	string_t src = (string_t)buffer;
	string_t dest = VID_START + 2*((VID_NCOLS*cursor.y) + cursor.x);
	count_t i = 0;

	// check start point is valid
	if (cursor.y >= VID_NROWS || cursor.x >= VID_NCOLS)
		return -1;

	while (*src)
	{
		++i;
		switch(*src)
		{
			case 0x0A:		// handle linefeeds
				dest = VID_START + 2*VID_NCOLS*(++(cursor.y));
				cursor.x = 0;
			case 0x0D:		// ignore carriage returns
				++src;
				break;
			case '\t':		// tab
				{
					count_t tomove = TAB_SPACES - (cursor.x % TAB_SPACES);
					if ((cursor.x + tomove) < VID_NCOLS)
					{
						while (tomove--)
						{
							*dest++ = ' ';
							*dest++ = cursor.attr;
							++(cursor.x);
						}
					}
					++src;
					break;
				}
			case '\b':		// backspace
				if (cursor.x)
				{
					dest -= 2;
					*dest++ = ' ';
					*dest = cursor.attr;
					--dest;
					--(cursor.x);
				}
				++src;
				break;
			default:		// normal characters
				*dest++ = *src++;
				*dest++ = cursor.attr;
				if (++(cursor.x) >= VID_NCOLS)
				{
					++(cursor.y);
					cursor.x = 0;
				}
		}
		if (cursor.y >= VID_NROWS)
		{
			VidScroll();
			--(cursor.y);
		}
	}
	VidUpdateCursor();
	return i;
}

// clear video memory
count_t VidClear()
{
	char_t* p = VID_START;
	count_t x, y;
	count_t i = 0;
	for (y = 0; y < VID_NROWS; y++)
	{
		for (x = 0; x < VID_NCOLS; x++)
		{
			*p++ = ' ';				// clear character
			*p++ = cursor.attr;		// set current attribute
			++i;					// increment counter
		}
	}
	cursor.x = cursor.y = 0;
	VidUpdateCursor();
	return i;
}

// scroll video memory
void VidScroll()
{
	count_t x;
	char_t* p = VID_START + 2*VID_NCOLS*(VID_NROWS-1);

	memcpy(VID_START, VID_START + 2*VID_NCOLS, 2*VID_NCOLS*(VID_NROWS-1));

	for (x = 0; x < VID_NCOLS; x++)
	{
		*p++ = ' ';				// clear character
		*p++ = cursor.attr;		// set current attribute
	}
}

// set current attribute
void VidSetAttr(byte_t attr)
{
	cursor.attr = attr;
}

// get current attribute
byte_t VidGetAttr()
{
	return cursor.attr;
}

void VidUpdateCursor()
{
    unsigned_t pos = (cursor.y * VID_NCOLS) + cursor.x;
    byte_t orig;

    // Save original contents of CRT address register.
    orig = inb_p(VID_AREG);

    // Set the high cursor location byte
    outb_p(VID_CURSOR_LOC_HIGH_REG, VID_AREG);
    outb_p((pos>>8) & 0xff, VID_DREG);

    // Set the low cursor location byte
    outb_p(VID_CURSOR_LOC_LOW_REG, VID_AREG);
    outb_p(pos & 0xff, VID_DREG);

    // Restore contents of the CRT address register
    outb(orig, VID_AREG);
}
